#!/usr/bin/env python
#
# Copyright (C) 2013 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test cron item removal
"""

import os
import sys

sys.path.insert(0, '../')

import unittest
from crontab import CronTab, PY3
try:
    from test import test_support
except ImportError:
    from test import support as test_support

if PY3:
    unicode = str

START_TAB = """
3 * * * * command1 # CommentID C
2 * * * * command2 # CommentID AAB
1 * * * * command3 # CommentID B3
"""

class RemovalTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.filenames = []
        self.crontab = CronTab(tab=START_TAB.strip())

    def test_01_remove(self):
        """Remove Item"""
        self.assertEqual(len(self.crontab), 3)
        self.crontab.remove(self.crontab.crons[0])
        self.assertEqual(len(self.crontab), 2)
        self.assertEqual(len(self.crontab.render()), 69)

    def test_02_remove_all(self):
        """Remove All"""
        self.crontab.remove_all()
        self.assertEqual(len(self.crontab), 0)
        self.assertEqual(unicode(self.crontab), '')

    def test_03_remove_cmd(self):
        """Remove all with Command"""
        self.crontab.remove_all(command='command2')
        self.assertEqual(len(self.crontab), 2)
        self.assertEqual(len(self.crontab.render()), 67)
        self.crontab.remove_all(command='command3')
        self.assertEqual(len(self.crontab), 1)
        self.assertEqual(len(self.crontab.render()), 33)

    def test_04_remove_id(self):
        """Remove all with Comment/ID"""
        self.crontab.remove_all(comment='CommentID B3')
        self.assertEqual(len(self.crontab), 2)
        self.assertEqual(len(self.crontab.render()), 68)

    def test_05_remove_date(self):
        """Remove all with Time Code"""
        self.crontab.remove_all(time='2 * * * *')
        self.assertEqual(len(self.crontab), 2)
        self.assertEqual(len(self.crontab.render()), 67)

    def test_05_remove_all_error(self):
        """Remove all with old arg"""
        with self.assertRaises(AttributeError):
            self.crontab.remove_all('command')

    def test_06_removal_of_none(self):
        """Remove all respects None as a possible value"""
        self.crontab[1].set_comment(None)
        self.crontab.remove_all(comment=None)
        self.assertEqual(len(self.crontab), 2)
        self.assertEqual(len(self.crontab.render()), 67)

    def test_07_removal_in_loop(self):
        """Remove items in a loop"""
        for job in self.crontab:
            self.crontab.remove(job)
        self.assertEqual(len(self.crontab), 0)

    def test_08_removal_write_loop(self):
        """Remove items in a loop and save each time"""
        filename = self.get_new_file('removal')
        crontab = CronTab()
        for i in range(10):
            crontab.new('test', comment=str(i))
            crontab.write(filename)
            crontab = CronTab(tabfile=filename)
            self.assertEqual(len(crontab), 1)
            self.assertNotEqual(str(crontab), '')

            crontab.remove_all(comment=str(i))
            crontab.write(filename)
            crontab = CronTab(tabfile=filename)
            self.assertEqual(len(crontab), 0)
            self.assertEqual(str(crontab), '')

    def test_09_removal_during_iter(self):
        crontab = CronTab()
        for x in range(0, 5, 1):
            job = crontab.new(command="cmd", comment="SAME_ID")
            job.setall("%d * * * *" % (x + 1))
        for item in crontab.find_comment("SAME_ID"):
            crontab.remove(item)
        self.assertEqual(len(crontab), 0)

    def test_11_remove_generator(self):
        """Remove jobs from the find generator"""
        tabs = self.crontab.find_command('command2')
        self.crontab.remove(tabs)
        self.assertEqual(len(self.crontab), 2)

    def test_12_remove_nonsense(self):
        """Fail to remove bad type"""
        self.assertRaises(TypeError, self.crontab.remove, 5)
        self.assertRaises(TypeError, self.crontab.remove, "foo")

    def get_new_file(self, name):
        """Gets a filename and records it for deletion"""
        this_dir = os.path.dirname(__file__)
        filename = os.path.join(this_dir, 'data', 'spool', name)
        self.filenames.append(filename)
        return filename

    def tearDown(self):
        for filename in self.filenames:
            if os.path.isfile(filename):
                os.unlink(filename)


if __name__ == '__main__':
    test_support.run_unittest(
       RemovalTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2019 Kyle Bakker <xelnaga.zealot@gmail.com>
#                    Martin Owens <doctormo@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test equality between CronItems
"""

import unittest
from crontab import CronTab

try:
    from test import test_support
except ImportError:
    from test import support as test_support

CRONTAB1 = """
    3 * * * * command1 # CommentID C
    2 * * * * command2 # CommentID AAB
    1 * * * * command3 # CommentID B3
"""

CRONTAB2 = """
    3 * * * * command1 # CommentID C
    2 * * * * command2 # CommentID AAB
    1 * * * * command3 # CommentID B3
    4 * * * * command5 # CommentID K
"""

class EqualityTestCase(unittest.TestCase):
    """ Testing equality against CronItems"""
    def setUp(self):
        self.crontab1 = CronTab(tab=CRONTAB1)
        self.crontab2 = CronTab(tab=CRONTAB2)

    def test_equality(self):
        """Test equality"""
        self.assertEqual(self.crontab1[0], self.crontab2[0])

    def test_inequality(self):
        """Test inequality"""
        self.assertNotEqual(self.crontab1[0], self.crontab2[1])

    def test_listdifference(self):
        """Test diference between lists"""
        self.assertEqual(set(self.crontab2) - set(self.crontab1), {self.crontab2[3]})

    def test_hash(self):
        """Test object hashing"""
        self.assertEqual(self.crontab1[0].__hash__(), hash((self.crontab1[0])))

if __name__ == '__main__':
    test_support.run_unittest(EqualityTestCase)



#!/usr/bin/env python
#
# Copyright (C) 2011 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab enumerations.
"""

import os
import sys

sys.path.insert(0, '../')

import unittest
from crontab import CronTab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

INITAL_TAB = """
* * * JAN     SAT          enums
* * * MAR-APR *            ranges
* * * *       MON,FRI,WED  multiples
* * * *       *            com1 # Comment One
* * * *       *            com2 # Comment One
"""

COMMANDS = [
    'enums',
    'ranges',
    'multiples',
    'com1', 'com2',
]

RESULT_TAB = """
* * * JAN SAT enums
* * * MAR-APR * ranges
* * * * MON,WED,FRI multiples
* * * * * com1 # Comment One
* * * * * com2 # Comment One
"""

class EnumTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = CronTab(tab=INITAL_TAB)

    def test_01_presevation(self):
        """All Entries Re-Rendered Correctly"""
        self.crontab.write()
        results = RESULT_TAB.split('\n')
        line_no = 0
        for line in self.crontab.intab.split('\n'):
            self.assertEqual(str(line), results[line_no])
            line_no += 1

    def test_02_simple_enum(self):
        """Simple Enumerations"""
        e = list(self.crontab.find_command('enums'))[0]
        self.assertEqual(e.month, 'JAN')
        self.assertEqual(e.month.render(True), '1')
        self.assertEqual(e.dow, 'SAT')
        self.assertEqual(e.dow.render(True), '6')

    def test_03_enum_range(self):
        """Enumeration Ranges"""
        e = list(self.crontab.find_command('ranges'))[0]
        self.assertEqual(e.month, 'MAR-APR')
        self.assertEqual(e.month.render(True), '3-4' )

    def test_04_sets(self):
        """Enumeration Sets"""
        e = list(self.crontab.find_command('multiples'))[0]
        self.assertEqual(e.dow, 'MON,WED,FRI')
        self.assertEqual(e.dow.render(True), '1,3,5' )

    def test_05_create(self):
        """Create by Enumeration"""
        job = self.crontab.new(command='new')
        job.month.on('JAN')
        job.dow.on('SUN')
        self.assertEqual(str(job), '* * * JAN SUN new')

    def test_06_create_range(self):
        """Created Enum Range"""
        job = self.crontab.new(command='new2')
        job.month.during('APR', 'NOV').every(2)
        self.assertEqual(str(job), '* * * APR-NOV/2 * new2')

    def test_07_create_set(self):
        """Created Enum Set"""
        job = self.crontab.new(command='new3')
        job.month.on('APR')
        job.month.also.on('NOV','JAN')
        self.assertEqual(str(job), '* * * JAN,APR,NOV * new3')

    def test_08_find_comment(self):
        """Comment Set"""
        jobs = list(self.crontab.find_comment('Comment One'))
        self.assertEqual(len(jobs), 2)
        for job in jobs:
            self.assertEqual(job.comment, 'Comment One')

if __name__ == '__main__':
    test_support.run_unittest(
       EnumTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2016 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test any internal scheduling
"""

import os
import sys

import unittest
import crontab
import logging
import string
import random

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

logger = logging.getLogger()
logger.level = logging.WARNING

try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)
COMMAND = os.path.join(TEST_DIR, 'data', 'crontest ')

class SchedulerTestCase(unittest.TestCase):
    """Test scheduling functions of CronTab."""
    def setUp(self):
        self.tab = crontab.CronTab()
        try:
            import croniter
        except ImportError:
            self.skipTest("Croniter not installed")

        self.handlers = []
        self.log = crontab.LOG
        self.log.setLevel(logging.DEBUG)
        self.log.propagate = False
        for handler in self.log.handlers:
            self.handlers.append(handler)
            self.log.removeHandler(handler)

        self.err = StringIO()
        self.handler = logging.StreamHandler(self.err)
        self.log.addHandler(self.handler)

    def tearDown(self):
        logger.removeHandler(self.handler)
        for handler in self.handlers:
            logger.addHandler(handler)
        #self.err.close()

    def assertLog(self, text):
        self.handler.flush()
        self.assertEqual(self.err.getvalue().strip(), text)

    def assertSchedule(self, slices, count, result):
        uid = random.choice(string.ascii_letters)
        self.tab.new(command=COMMAND + '-h ' + uid).setall(slices)
        ret = list(self.tab.run_scheduler(count, cadence=0.01, warp=True))
        self.assertEqual(len(ret), result)
        if count > 0:
            self.assertEqual(ret[0], '-h|' + uid)

    def test_01_run(self):
        """Run the command"""
        self.tab.env['SHELL'] = crontab.SHELL
        ret = self.tab.new(command=COMMAND+'-h A').run()
        self.assertEqual(ret, '-h|A')

    def test_02_run_error(self):
        """Run with errors"""
        ret = self.tab.new(command=COMMAND+'-e B').run()
        self.assertEqual(ret, '')
        self.assertLog('-e|B')

    def test_03_schedule(self):
        """Simple Schedule"""
        self.assertSchedule("* * * * *", 5, 4)

    def test_04_schedule_ten(self):
        """Every Ten Minutes"""
        self.assertSchedule("*/10 * * * *", 12, 1)

    def test_05_env_passed(self):
        """Environment is passed in"""
        self.tab.env['CR_VAR'] = 'BABARIAN'
        ret = self.tab.new(command=COMMAND+' -ev').run()
        self.assertEqual(ret, 'BABARIAN')

if __name__ == '__main__':
    test_support.run_unittest(SchedulerTestCase)



#!/usr/bin/env python
#
# Copyright (C) 2013 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test simple 'every' api.
"""

import os
import sys

sys.path.insert(0, '../')

import unittest
from crontab import CronTab, PY3
try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)

if PY3:
    unicode = str

class EveryTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = CronTab(tabfile=os.path.join(TEST_DIR, 'data', 'test.tab'))

    def test_00_minutes(self):
        """Every Minutes"""
        for job in self.crontab:
            job.every(3).minutes()
            self.assertEqual(job.slices.clean_render(), '*/3 * * * *')
            job.minutes.every(5)
            self.assertEqual(job.slices.clean_render(), '*/5 * * * *')

    def test_01_hours(self):
        """Every Hours"""
        for job in self.crontab:
            job.every(3).hours()
            self.assertEqual(job.slices.clean_render(), '0 */3 * * *')

    def test_02_dom(self):
        """Every Day of the Month"""
        for job in self.crontab:
            job.every(3).dom()
            self.assertEqual(job.slices.clean_render(), '0 0 */3 * *')

    def test_03_single(self):
        """Every Single Hour"""
        for job in self.crontab:
            job.every().hour()
            self.assertEqual(job.slices.clean_render(), '0 * * * *')

    def test_04_month(self):
        """Every Month"""
        for job in self.crontab:
            job.every(3).months()
            self.assertEqual(job.slices.clean_render(), '0 0 1 */3 *')

    def test_05_dow(self):
        """Every Day of the Week"""
        for job in self.crontab:
            job.every(3).dow()
            self.assertEqual(job.slices.clean_render(), '0 0 * * */3')

    def test_06_year(self):
        """Every Year"""
        for job in self.crontab:
            job.every().year()
            self.assertEqual(job.slices.render(), '@yearly')
            self.assertEqual(job.slices.clean_render(), '0 0 1 1 *')
            self.assertRaises(ValueError, job.every(2).year)

    def test_07_reboot(self):
        """Every Reboot"""
        for job in self.crontab:
            job.every_reboot()
            self.assertEqual(job.slices.render(), '@reboot')
            self.assertEqual(job.slices.clean_render(), '* * * * *')

    def test_08_newitem(self):
        """Every on New Item"""
        job = self.crontab.new(command='hourly')
        job.every().hour()
        self.assertEqual(job.slices.render(), '@hourly')
        job = self.crontab.new(command='firstly')
        job.hours.every(2)
        self.assertEqual(job.slices.render(), '* */2 * * *')

if __name__ == '__main__':
    test_support.run_unittest(
       EveryTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2016 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test optional crontab description intergration.
"""

import sys
sys.path.insert(0, '../')

import unittest
import crontab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

INITAL_TAB = """
# Basic Comment
20 10-20 */2 * * execute # At 20 minutes past the hour, between 10:00 AM and 08:59 PM, every 2 days
"""

class DescriptorTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = crontab.CronTab(tab=INITAL_TAB)
        self.job = list(self.crontab.find_command('execute'))[0]
        try:
            import cron_descriptor
        except ImportError:
            self.skipTest("Cron-descriptor module not installed")

    def test_00_no_module(self):
        """No module found"""
        # Remove module if imported already
        for i in range(1, 4):
            name = '.'.join((['cron_descriptor'] * i))
            if name in sys.modules:
                del sys.modules[name]

        old, sys.path = sys.path, []
        with self.assertRaises(ImportError):
            self.job.description()
        sys.path = old

    def test_01_description(self):
        """Get Job Description"""
        self.assertEqual(self.job.description(), self.job.comment)

if __name__ == '__main__':
    test_support.run_unittest(DescriptorTestCase)




#!/usr/bin/env python
#
# Copyright (C) 2012 Jay Sigbrandt <jsigbrandt@slb.com>
#                    Martin Owens <doctormo@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab usage.
"""

import os
import sys

import unittest
import crontab

from datetime import date, time, datetime, timedelta

try:
    from test import test_support
except ImportError:
    from test import support as test_support

crontab.LOG.setLevel(crontab.logging.ERROR)
TEST_DIR = os.path.dirname(__file__)

class DummyStdout(object):
    def write(self, text):
        pass

BASIC = '@hourly firstcommand\n\n'
USER = '\n*/4 * * * * user_command # user_comment\n\n\n'
crontab.CRON_COMMAND = "%s %s" % (sys.executable, os.path.join(TEST_DIR, 'data', 'crontest'))

def flush():
    pass

class Attribute(object):
    def __init__(self, obj, attr, value):
        self.obj = obj
        self.attr = attr
        self.value = value

    def __enter__(self, *args, **kw):
        if hasattr(self.obj, self.attr):
            self.previous = getattr(self.obj, self.attr)
        setattr(self.obj, self.attr, self.value)

    def __exit__(self, *args, **kw):
        if hasattr(self, 'previous'):
            setattr(self.obj, self.attr, self.previous)
        else:
            delattr(self.obj, self.attr)


class UseTestCase(unittest.TestCase):
    """Test use documentation in crontab."""
    def setUp(self):
        self.filenames = []

    def test_01_empty(self):
        """Open system crontab"""
        cron = crontab.CronTab()
        self.assertEqual(cron.render(), "")
        self.assertEqual(cron.__unicode__(), "")
        self.assertEqual(repr(cron), "<Unattached CronTab>")

    def test_02_user(self):
        """Open a user's crontab"""
        cron = crontab.CronTab(user='basic')
        self.assertEqual(cron.render(), BASIC)
        self.assertEqual(repr(cron), "<User CronTab 'basic'>")

    def test_03_usage(self):
        """Dont modify crontab"""
        cron = crontab.CronTab(tab='')
        sys.stdout = DummyStdout()
        sys.stdout.flush = flush
        try:
            exec(crontab.__doc__)
        except ImportError:
            pass
        sys.stdout = sys.__stdout__
        self.assertEqual(cron.render(), '')

    def test_04_username(self):
        """Username is True"""
        cron = crontab.CronTab(user=True)
        self.assertNotEqual(cron.user, True)
        self.assertEqual(cron.render(), USER)
        self.assertEqual(repr(cron), "<My CronTab>")

    def test_05_nouser(self):
        """Username doesn't exist"""
        cron = crontab.CronTab(user='nouser')
        self.assertEqual(cron.render(), '')

    def test_06_touser(self):
        """Write to user API"""
        cron = crontab.CronTab(tab=USER)
        self.assertEqual(repr(cron), "<Unattached CronTab>")
        cron.write_to_user('bob')
        filename = os.path.join(TEST_DIR, 'data', 'spool', 'bob')
        self.filenames.append(filename)
        self.assertTrue(os.path.exists(filename))
        self.assertEqual(repr(cron), "<User CronTab 'bob'>")

    def test_06_touser_fail(self):
        """Write to user failed"""
        cron = crontab.CronTab(tab=USER)
        self.assertRaises(IOError, cron.write_to_user, 'john')
        filename = os.path.join(TEST_DIR, 'data', 'spool', 'john')
        if os.path.exists(filename):
            os.unlink(filename)
            self.assertTrue(filename)


    def test_07_ioerror_read(self):
        """No filename ioerror"""
        with self.assertRaises(IOError):
            cron = crontab.CronTab(user='error')
            cron.read()

    def test_07_ioerror_write(self):
        """User not specified, nowhere to write to"""
        cron = crontab.CronTab()
        with self.assertRaises(IOError):
            cron.write()

    def test_08_cronitem(self):
        """CronItem Standalone"""
        item = crontab.CronItem.from_line('noline')
        self.assertTrue(item.is_enabled())
        with self.assertRaises(UnboundLocalError):
            item.delete()
        item.set_command('nothing')
        self.assertEqual(item.render(), '* * * * * nothing')

    def test_10_time_object(self):
        """Set slices using time object"""
        item = crontab.CronItem(command='cmd')
        self.assertEqual(str(item.slices), '* * * * *')
        item.setall(time(1, 2))
        self.assertEqual(str(item.slices), '2 1 * * *')
        self.assertTrue(item.is_valid())
        item.setall(time(0, 30, 0, 0))
        self.assertEqual(str(item.slices), '30 0 * * *')
        self.assertTrue(item.is_valid())
        self.assertEqual(str(item), '30 0 * * * cmd')

    def test_11_date_object(self):
        """Set slices using date object"""
        item = crontab.CronItem(command='cmd')
        self.assertEqual(str(item.slices), '* * * * *')
        item.setall(date(2010, 6, 7))
        self.assertEqual(str(item.slices), '0 0 7 6 *')
        self.assertTrue(item.is_valid())

    def test_12_datetime_object(self):
        """Set slices using datetime object"""
        item = crontab.CronItem(command='cmd')
        self.assertEqual(str(item.slices), '* * * * *')
        item.setall(datetime(2009, 8, 9, 3, 4))
        self.assertTrue(item.is_valid())
        self.assertEqual(str(item.slices), '4 3 9 8 *')

    def test_20_slice_validation(self):
        """CronSlices class and objects can validate"""
        CronSlices = crontab.CronSlices
        self.assertTrue(CronSlices('* * * * *').is_valid())
        self.assertTrue(CronSlices.is_valid('* * * * *'))
        self.assertTrue(CronSlices.is_valid('*/2 * * * *'))
        self.assertTrue(CronSlices.is_valid('* 1,2 * * *'))
        self.assertTrue(CronSlices.is_valid('* * 1-5 * *'))
        self.assertTrue(CronSlices.is_valid('* * * * MON-WED'))
        self.assertTrue(CronSlices.is_valid('@reboot'))

        sliced = CronSlices('* * * * *')
        sliced[0].parts = [300]
        self.assertEqual(str(sliced), '300 * * * *')
        self.assertFalse(sliced.is_valid())
        self.assertFalse(CronSlices.is_valid('P'))
        self.assertFalse(CronSlices.is_valid('*/61 * * * *'))
        self.assertFalse(CronSlices.is_valid('* 1,300 * * *'))
        self.assertFalse(CronSlices.is_valid('* * 50-1 * *'))
        self.assertFalse(CronSlices.is_valid('* * * * FRO-TOO'))
        self.assertFalse(CronSlices.is_valid('@retool'))

    def test_21_slice_special(self):
        """Rendering can be done without specials"""
        cronitem = crontab.CronItem('true')
        cronitem.setall('0 0 * * *')
        self.assertEqual(cronitem.render(specials=True), '@daily true')
        self.assertEqual(cronitem.render(specials=None), '0 0 * * * true')
        cronitem.setall('@daily')
        self.assertEqual(cronitem.render(specials=None), '@daily true')
        self.assertEqual(cronitem.render(specials=False), '0 0 * * * true')

    def test_25_open_pipe(self):
        """Test opening pipes"""
        from crontab import open_pipe, CRON_COMMAND
        pipe = open_pipe(CRON_COMMAND, h=None, a='one', abc='two')
        (out, err) = pipe.communicate()
        self.assertEqual(err, b'')
        self.assertEqual(out, b'--abc=two|-a|-h|one\n')

    def test_07_zero_padding(self):
        """Can we get zero padded output"""
        cron = crontab.CronTab(tab="02 3-5 2,4 */2 01 cmd")
        self.assertEqual(str(cron), '2 3-5 2,4 */2 1 cmd\n')
        with Attribute(crontab, 'ZERO_PAD', True):
            self.assertEqual(str(cron), '02 03-05 02,04 */2 01 cmd\n')

    def test_08_reset_after_daily(self):
        """Can jobs be rescheduled after midnight"""
        cron = crontab.CronTab(tab="@daily cmd")
        job = cron[0]
        self.assertEqual(str(job), '@daily cmd')
        job.minute.on(5)
        job.hour.on(1)
        self.assertEqual(str(job), '5 1 * * * cmd')

    def test_09_pre_comment(self):
        """Test use of pre_comments"""
        text = """
# Unattached comment

5 * * * 6 some_command
# Attached comment
*/5 * * * * another_command
*/5 * * * * my_command # Other comment
"""
        tab = crontab.CronTab(tab=text)
        self.assertEqual(tab[0].comment, "")
        self.assertEqual(tab[1].comment, "Attached comment")
        self.assertEqual(tab[2].comment, "Other comment")
        self.assertEqual(tab.render(), text)

    def tearDown(self):
        for filename in self.filenames:
            if os.path.exists(filename):
                os.unlink(filename)


if __name__ == '__main__':
    test_support.run_unittest(
       UseTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2017 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test the creation, reading and writing on environment variables.
"""

import os
import sys
sys.path.insert(0, '../')

from .test_usage import TEST_DIR

import unittest
from collections import OrderedDict

from crontab import CronTab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

INITIAL_TAB = """PERSONAL_VAR=bar

CRON_VAR=fork
34 12 * * * eat_soup

CRON_VAR=spoon
35 12 * * * eat_salad

CRON_VAR=knife
36 12 * * * eat_icecream

SECONDARY=fork
38 12 * * * eat_steak
"""

class EnvTestCase(unittest.TestCase):
    """Test vixie cron user addition."""
    def setUp(self):
        self.crontab = CronTab(tab=INITIAL_TAB)

    def test_01_consistancy(self):
        """Read in crontab and write out the same"""
        self.assertEqual(INITIAL_TAB, str(self.crontab))

    def test_02_top_vars(self):
        """Whole file env variables"""
        crontab = CronTab(tab="SHELL=dash\n")
        self.assertEqual(str(crontab), "SHELL=dash\n")
        self.assertEqual(crontab.env['SHELL'], 'dash')
        crontab.env['SHELL'] = 'bash'
        self.assertEqual(str(crontab), "SHELL=bash\n")
        self.assertEqual(crontab.env['SHELL'], 'bash')

    def test_03_get_job_var(self):
        """Test each of the job env structures"""
        for job, expected in zip(self.crontab, [
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'fork'},
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'spoon'},
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'knife'},
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'knife', 'SECONDARY': 'fork'},
          ]):
            self.assertEqual(OrderedDict(job.env.all()), expected)

    def test_04_set_job_var(self):
        """Test that variables set are applied correctly"""
        self.crontab[1].env['CRON_VAR'] = 'javlin'
        self.assertEqual(self.crontab[1].env.all(),
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'javlin'})
        self.crontab[2].env['CRON_VAR'] = 'javlin'
        self.assertEqual(self.crontab[2].env.all(),
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'javlin'})
        self.assertEqual(self.crontab[3].env['PERSONAL_VAR'], 'bar')
        self.assertEqual(self.crontab[3].env.all(),
            {'PERSONAL_VAR': 'bar', 'CRON_VAR': 'javlin', 'SECONDARY': 'fork'})

        self.crontab.env['PERSONAL_VAR'] = 'foo'
        self.assertEqual(self.crontab[2].env.all(),
            {'PERSONAL_VAR': 'foo', 'CRON_VAR': 'javlin'})

        self.crontab.env['CRON_VAR'] = 'fork'
        self.assertEqual(self.crontab[0].env.all(),
            {'PERSONAL_VAR': 'foo', 'CRON_VAR': 'fork'})

        self.assertEqual(str(self.crontab), """PERSONAL_VAR=foo
CRON_VAR=fork

34 12 * * * eat_soup

CRON_VAR=javlin
35 12 * * * eat_salad

36 12 * * * eat_icecream

SECONDARY=fork
38 12 * * * eat_steak
""")

    def test_05_no_env(self):
        """Test that we get an error asking for no var"""
        with self.assertRaises(KeyError):
            self.crontab.env['BLUE_BOTTLE']
        with self.assertRaises(KeyError):
            self.crontab[0].env['RED_BOTTLE']

    def test_06_env_access(self):
        cron = CronTab(tab="""
MYNAME='Random'

* * * * * echo "first: $MYNAME"
* * * * * echo "second: $MYNAME"
* * * * * echo "third: $MYNAME"
        """)
        for job in cron:
            self.assertEqual(job.env['MYNAME'], "Random")

    def test_07_mutated_dict(self):
        """Test when the ordered dict is changed during loop"""
        cron = CronTab(tab="""
ALL='all'
ABCD='first'
* * * * * echo "first"
        """)

    def test_08_space_quotes(self):
        """Test that spaces and quotes are handled correctly"""
        cron = CronTab(tab="""
A=   123   
B="   123   "
C='   123   '
D=  " 123 "  
E= 1 2 3 
""")
        self.assertEqual(cron.env['A'], '123')
        self.assertEqual(cron.env['B'], '   123   ')
        self.assertEqual(cron.env['C'], '   123   ')
        self.assertEqual(cron.env['D'], ' 123 ')
        self.assertEqual(cron.env['E'], '1 2 3')

        self.assertEqual(str(cron), """A=123
B="   123   "
C="   123   "
D=" 123 "
E="1 2 3"

""")

    def test_09_delete_middle(self):
        """Test that a delete doesn't remove vars"""
        self.crontab.remove_all(command='eat_icecream')
        self.crontab.remove_all(command='eat_soup')
        self.assertEqual(str(self.crontab), """PERSONAL_VAR=bar
CRON_VAR=spoon
35 12 * * * eat_salad


CRON_VAR=knife
SECONDARY=fork
38 12 * * * eat_steak
""")

    def test_10_empty_env(self):
        """Test when an env is an empty string it should have quotes"""
        tab='MAILTO=""\n'
        self.assertEqual(str(CronTab(tab=tab)), tab)

    def test_11_empty_flow(self):
        """Test what happends when an env is involved in flow"""
        tab = """
# A
# B
MAILTO=""
# C
*/10 * * * * /home/pi/job.py # any job
"""
        cron = CronTab(tab=tab)
        job = cron.new('update.py', 'update')
        job.setall('1 12 * * 3')
        self.assertTrue(job.is_valid())
        self.assertEqual(str(cron), """MAILTO=""

# A
# B
# C
*/10 * * * * /home/pi/job.py # any job

1 12 * * 3 update.py # update
""")

        cron.remove_all(comment='update')

        cron.write_to_user("bob")
        filename = os.path.join(TEST_DIR, 'data', 'spool', 'bob')
        with open(filename, 'r') as fhl:
            self.assertEqual(fhl.read(), """MAILTO=""

# A
# B
# C
*/10 * * * * /home/pi/job.py # any job
""")
        os.unlink(filename)

if __name__ == '__main__':
    test_support.run_unittest(EnvTestCase)



#!/usr/bin/env python
#
# Copyright (C) 2013 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test croniter extention to find out when items will next happen.
"""

import os
import sys
from datetime import datetime

sys.path.insert(0, '../')

import unittest
import crontab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

INITAL_TAB = """
# Basic Comment
20 * * * * execute # comment
"""

class CroniterTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = crontab.CronTab(tab=INITAL_TAB)
        self.job = list(self.crontab.find_command('execute'))[0]
        try:
            import croniter
        except ImportError:
            self.skipTest("Croniter not installed")

    def test_00_nocroniter(self):
        """No Croniter"""
        # Remove croniter if imported before.
        for i in range(1, 4):
            name = '.'.join((['croniter'] * i))
            if name in sys.modules:
                del sys.modules[name]

        old, sys.path = sys.path, []
        with self.assertRaises(ImportError):
            self.job.schedule(datetime(2001, 10, 11, 1, 12, 10))
        sys.path = old

    def test_01_schedule(self):
        """Get Scheduler"""
        ct = self.job.schedule(datetime(2009, 10, 11, 5, 12, 10))
        self.assertTrue(ct)

    def test_02_next(self):
        """Get Next Scheduled Items"""
        ct = self.job.schedule(datetime(2000, 10, 11, 5, 12, 10))
        self.assertEqual(ct.get_next(), datetime(2000, 10, 11, 5, 20, 0))
        self.assertEqual(ct.get_next(), datetime(2000, 10, 11, 6, 20, 0))

    def test_03_prev(self):
        """Get Prev Scheduled Items"""
        ct = self.job.schedule(datetime(2001, 10, 11, 1, 12, 10))
        self.assertEqual(ct.get_prev(), datetime(2001, 10, 11, 0, 20, 0))
        self.assertEqual(ct.get_prev(), datetime(2001, 10, 10, 23, 20, 0))

    def test_04_current(self):
        """Get Current Item"""
        ct = self.job.schedule(datetime(2001, 10, 11, 1, 12, 10))
        self.assertEqual(ct.get_current(), datetime(2001, 10, 11, 1, 12, 10))
        self.assertEqual(ct.get_current(), datetime(2001, 10, 11, 1, 12, 10))

if __name__ == '__main__':
    test_support.run_unittest(
       CroniterTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2008-2018 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
# pylint: disable=bad-whitespace
"""Setup the crontab module"""

import os
from setuptools import setup
from crontab import __version__, __pkgname__

# remove MANIFEST. distutils doesn't properly update it when the
# contents of directories change.
if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

# Grab description for Pypi
with open('README.rst') as fhl:
    description = fhl.read()

# Used for rpm building
RELEASE = "1"

setup(
    name             = __pkgname__,
    version          = __version__,
    release          = RELEASE,
    description      = 'Python Crontab API',
    long_description = description,
    author           = 'Martin Owens',
    url              = 'https://gitlab.com/doctormo/python-crontab/',
    author_email     = 'doctormo@gmail.com',
    test_suite       = 'tests',
    platforms        = 'linux',
    license          = 'LGPLv3',
    py_modules       = ['crontab', 'crontabs', 'cronlog'],
    provides         = ['crontab', 'crontabs', 'cronlog'],
    install_requires = ['python-dateutil'],
    extras_require   = {
        'cron-schedule': ['croniter'],
        'cron-description': ['cron-descriptor'],
    },
    classifiers      = [
        'Development Status :: 5 - Production/Stable',
        'Development Status :: 6 - Mature',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Operating System :: POSIX :: SunOS/Solaris',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    options = {
        'bdist_rpm': {
            'build_requires': [
                'python',
                'python-setuptools',
            ],
            'release': RELEASE,
        },
    },
)



#!/usr/bin/env python
#
# Copyright (C) 2013 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab interaction.
"""

import re
import os
import unittest

from crontab import CronTab, CronSlices, CronSlice, PY3
from .utils import LoggingMixin

try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)

if PY3:
    unicode = str #pylint: disable=redefined-builtin

COMMANDS = [
    'firstcommand',
    'range',
    'byweek',
    'disabled',
    'spaced',
    'python /example_app/testing.py',
    'rebooted',
    'echo % escaped',
    'cat %%',
]

RESULT_TAB = """# First Comment
# Edit this line to test for mistaken checks

# m h dom mon dow user command

*/30 * * * * firstcommand
* 10-20/3 * * * range
# Middle Comment
* * * 10 * byweek # Comment One
# * * * * * disabled
0 5 * * * spaced # Comment  Two

# monitorCron\\n*/59 * * * * python /example_app/testing.py

@reboot rebooted # re-id
# Last Comment @has this # extra

# percent_one\\n* * * * * echo \% escaped
# percent_two\\n* * * * * cat \%\% % stdin%with%lines
"""

class InteractionTestCase(LoggingMixin, unittest.TestCase):
    """Test basic functionality of crontab."""
    log_name = 'crontab'

    def setUp(self):
        super(InteractionTestCase, self).setUp()
        self.crontab = CronTab(tabfile=os.path.join(TEST_DIR, 'data', 'test.tab'))

    def test_01_presevation(self):
        """All Entries Re-Rendered Correctly"""
        results = RESULT_TAB.split('\n')
        for line_no, line in enumerate(self.crontab.lines):
            self.assertEqual(str(line), results[line_no].replace('\\n', '\n'))

    def test_02_access(self):
        """All Entries Are Accessable"""
        for line_no, job in enumerate(self.crontab):
            self.assertEqual(str(job.command), COMMANDS[line_no])
        else: # pylint: disable=useless-else-on-loop
            self.assertEqual(line_no + 1, 9)

    def test_03_blank(self):
        """Render Blank"""
        job = self.crontab.new(command='blank')
        self.assertEqual(repr(job), '<CronItem \'* * * * * blank\'>')
        self.assertEqual(job.render(), '* * * * * blank')

    def test_04_number(self):
        """Render Number"""
        job = self.crontab.new(command='number')
        job.minute.on(4)
        self.assertEqual(job.render(), '4 * * * * number')

    def test_05_fields(self):
        """Render Hours Days and Weeks"""
        job = self.crontab.new(command='fields')
        job.hour.on(4)
        self.assertEqual(job.render(), '* 4 * * * fields')
        job.dom.on(5)
        self.assertEqual(job.render(), '* 4 5 * * fields')
        job.month.on(6)
        self.assertEqual(job.render(), '* 4 5 6 * fields')
        job.dow.on(7)
        self.assertEqual(job.render(), '* 4 5 6 0 fields')

    def test_05_multiple_fields(self):
        """Test mutliple named fields"""
        job = self.crontab.new(command='fields')
        job.hour.on(4, 6, 7)
        self.assertEqual(job.render(), '* 4,6,7 * * * fields')
        job.dow.on('SUN', 'FRI')
        self.assertEqual(job.render(), '* 4,6,7 * * SUN,FRI fields')

    def test_06_clear(self):
        """Render Hours Days and Weeks"""
        job = self.crontab.new(command='clear')
        job.minute.on(3)
        job.hour.on(4)
        job.dom.on(5)
        job.month.on(6)
        job.dow.on(7)
        self.assertEqual(job.render(), '3 4 5 6 0 clear')
        job.clear()
        self.assertEqual(job.render(), '* * * * * clear')

    def test_07_range(self):
        """Render Time Ranges"""
        job = self.crontab.new(command='range')
        job.minute.during(4, 10)
        self.assertEqual(job.render(), '4-10 * * * * range')
        job.minute.during(15, 19)
        self.assertEqual(job.render(), '15-19 * * * * range')
        job.minute.clear()
        self.assertEqual(job.render(), '* * * * * range')
        job.minute.during(15, 19)
        self.assertEqual(job.render(), '15-19 * * * * range')
        job.minute.also.during(4, 10)
        self.assertEqual(job.render(), '4-10,15-19 * * * * range')

    def test_08_sequence(self):
        """Render Time Sequences"""
        job = self.crontab.new(command='seq')
        job.hour.every(4)
        self.assertEqual(job.render(), '* */4 * * * seq')
        job.hour.during(2, 10)
        self.assertEqual(job.render(), '* 2-10 * * * seq')
        job.hour.clear()
        self.assertEqual(job.render(), '* * * * * seq')
        job.hour.during(2, 10).every(4)
        self.assertEqual(job.render(), '* 2-10/4 * * * seq')
        job.hour.also.during(1, 4)
        self.assertEqual(job.render(), '* 1-4,2-10/4 * * * seq')
        job.hour.also.every(4)
        self.assertEqual(job.render(), '* */4,1-4,2-10/4 * * * seq')

    def test_10_comment(self):
        """Render cron Comments"""
        job = self.crontab.new(command='com', comment='I love this')
        self.assertEqual(unicode(job), '* * * * * com # I love this')

    def test_11_disabled(self):
        """Disabled Job"""
        jobs = list(self.crontab.find_command('firstcommand'))
        self.assertTrue(jobs[0].enabled)
        jobs = list(self.crontab.find_command('disabled'))
        self.assertFalse(jobs[0].enabled)

    def test_12_disable(self):
        """Disable and Enable Job"""
        job = self.crontab.new(command='dis')
        job.enable(False)
        self.assertEqual(unicode(job), '# * * * * * dis')
        job.enable()
        self.assertEqual(unicode(job), '* * * * * dis')

    def test_13_plural(self):
        """Plural API"""
        job = self.crontab.new(command='plural')
        job.minutes.every(4)
        job.hours.on(5, 6)
        job.day.on(4)
        job.months.on(2)
        self.assertEqual(unicode(job), '*/4 5,6 4 2 * plural')

    def test_14_valid(self):
        """Valid and Invalid"""
        job = self.crontab.new(command='valid')
        job.minute.every(2)
        job.valid = False
        with self.assertRaises(ValueError):
            str(job)
        # Disabled jobs still work
        job.enabled = False
        str(job)

        job.enabled = True
        with self.assertRaises(ValueError):
            self.crontab.render(errors=True)
        self.crontab.render(errors=False)
        self.assertFalse(job.enabled)

        job = self.crontab.new()
        self.assertFalse(job.is_valid())
        job.set_command("now_valid")
        self.assertTrue(job.is_valid())

    def test_15_slices(self):
        """Invalid Slices"""
        mon = CronSlices('* * * * *')
        with self.assertRaises(ValueError):
            CronSlices('* * * */15 *')
        with self.assertRaises(AssertionError):
            mon.setall(mon)

    def test_16_slice(self):
        """Single Slice"""
        dow = CronSlice({'name': 'M', 'max': 7, 'min': 0, 'enum': ['a']}, '*/6')
        self.assertEqual(repr(dow), '<CronSlice \'*/6\'>')

        self.assertEqual(repr(dow.parse_value('a')), 'a')
        with self.assertRaises(ValueError):
            dow.parse_value('b')

        self.assertEqual(dow.get_range()[0].render(), '*')
        with self.assertRaises(ValueError):
            dow.get_range('%')

    def test_17_slice_id(self):
        """Single slice by Id"""
        self.assertEqual(CronSlice(1).max, 23)

    def test_18_range_cmp(self):
        """Compare ranges"""
        dow = CronSlice({'max': 5, 'min': 0})
        three = dow.get_range(2, 4)[0]
        self.assertGreater(three, 2)
        self.assertLess(three, 4)
        self.assertEqual(str(three), '2-4')

    def test_18_find(self):
        """Find a command and comments by name"""
        cmds = list(self.crontab.find_command('byweek'))
        self.assertEqual(len(cmds), 1)
        self.assertEqual(cmds[0].comment, 'Comment One')

        cmds = list(self.crontab.find_command(re.compile(r'stc\w')))
        self.assertEqual(len(cmds), 1)
        self.assertEqual(cmds[0].command, 'firstcommand')

        cmds = list(self.crontab.find_command('Comment One'))
        self.assertEqual(len(cmds), 0)

        cmds = list(self.crontab.find_comment('Comment One'))
        self.assertEqual(len(cmds), 1)
        self.assertEqual(cmds[0].command, 'byweek')

        cmds = list(self.crontab.find_comment(re.compile(r'om+en\w O')))
        self.assertEqual(len(cmds), 1)
        self.assertEqual(cmds[0].comment, 'Comment One')

        cmds = list(self.crontab.find_comment(re.compile(r'stc\w')))
        self.assertEqual(len(cmds), 0)

    def test_20_write(self):
        """Write CronTab to file"""
        self.crontab.write('output.tab')
        self.assertTrue(os.path.exists('output.tab'))
        os.unlink('output.tab')

    def test_21_multiuse(self):
        """Multiple Renderings"""
        cron = '# start of tab\n'
        for i in range(10):
            crontab = CronTab(tab=cron)
            p = list(crontab.new(command='multi%d' % i))
            cron = unicode(crontab)
            crontab = CronTab(tab=cron)
            list(crontab.find_command('multi%d' % i))[0].delete()
            cron = unicode(crontab)
        self.assertEqual(unicode(crontab), '# start of tab\n')

    def test_22_min(self):
        """Minimum Field Values"""
        job = self.crontab.new(command='min')
        job.minute.on('<')
        job.hour.on('<')
        job.dom.on('<')
        job.month.on('<')
        self.assertEqual(unicode(job), '@yearly min')

    def test_23_max(self):
        """Maximum Field Values"""
        job = self.crontab.new(command='max')
        job.minute.on('>')
        job.hour.on('>')
        job.dom.on('>')
        job.month.on('>')
        self.assertEqual(unicode(job), '59 23 31 12 * max')

    def test_24_special_r(self):
        """Special formats are read and written"""
        tab = CronTab(tabfile=os.path.join(TEST_DIR, 'data', 'specials_enc.tab'))
        self.assertEqual(tab.render(), """@hourly hourly\n@daily daily\n@midnight midnight\n@weekly weekly\n@reboot reboot\n""")
        self.assertEqual(len(list(tab)), 5)

    def test_24_special_d(self):
        """Removal All Specials"""
        tab = CronTab(tabfile=os.path.join(TEST_DIR, 'data', 'specials.tab'))
        tab.remove_all()
        self.assertEqual(len(list(tab)), 0)

    def test_24_special_w(self):
        """Write Specials"""
        tab = CronTab(tabfile=os.path.join(TEST_DIR, 'data', 'specials.tab'))
        self.assertEqual(tab.render(), """@hourly hourly\n@daily daily\n@weekly weekly\n""")
        self.assertEqual(len(list(tab)), 3)

    def test_25_setall(self):
        """Set all values at once"""
        job = self.crontab.new(command='all')
        job.setall(1, '*/2', '2-4', '>', 'SUN')
        self.assertEqual(unicode(job), '1 */2 2-4 12 SUN all')
        job.setall('*/2')
        self.assertEqual(unicode(job), '*/2 * * * * all')
        job.setall('1 */2 2-4 12 SUN')
        self.assertEqual(unicode(job), '1 */2 2-4 12 SUN all')
        job.setall(['*'])
        self.assertEqual(unicode(job), '* * * * * all')

    def test_26_setall_obj(self):
        """Copy all values"""
        job = self.crontab.new(command='all')
        job2 = self.crontab.new(command='ignore')
        job2.setall("1 */2 2-4 12 SUN")
        job.setall(job2)
        self.assertEqual(unicode(job), '1 */2 2-4 12 SUN all')
        job2.setall("2 */3 4-8 10 MON")
        job.setall(job2.slices)
        self.assertEqual(unicode(job), '2 */3 4-8 10 MON all')

    def test_27_commands(self):
        """Get all commands"""
        self.assertEqual(list(self.crontab.commands),
                         [u'firstcommand', u'range', u'byweek', u'disabled',
                          u'spaced', u'python /example_app/testing.py',
                          u'rebooted',
                          u'echo % escaped',
                          u'cat %%'
                          ])

    def test_28_comments(self):
        """Get all comments"""
        self.assertEqual(list(self.crontab.comments),
                         ['Comment One', 'Comment  Two', 'monitorCron',
                          're-id', 'percent_one', 'percent_two'])

    def test_29_set_lines(self):
        """Set crontab lines directly"""
        cron = CronTab(tab='')
        cron.lines = [
            str(self.crontab.lines[5]),
        ]
        self.assertEqual(str(cron), '\n*/30 * * * * firstcommand\n')

        with self.assertRaises(AttributeError):
            cron.crons = ['A']


if __name__ == '__main__':
    test_support.run_unittest(InteractionTestCase)



#!/usr/bin/env python
#
# Copyright (C) 2012 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab interaction.
"""

import os
import sys

import unittest
import crontab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)

INITAL_TAB = """
# First Comment
0,30 * * * * firstcommand
"""

class UserTestCase(unittest.TestCase):
    def test_01_self_user(self):
        tab = crontab.CronTab(user='foo')
        self.assertEqual(tab.user_opt, {'u': 'foo'})
        tab = crontab.CronTab(user=crontab.current_user())
        self.assertEqual(tab.user_opt, {})


class CompatTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    @classmethod
    def setUpClass(cls):
        crontab.SYSTEMV = True

    @classmethod
    def tearDownClass(cls):
        crontab.SYSTEMV = False

    def setUp(self):
        self.crontab = crontab.CronTab(tab=INITAL_TAB)

    def test_00_enabled(self):
        """Test Compatability Mode"""
        self.assertTrue(crontab.SYSTEMV)

    def test_01_addition(self):
        """New Job Rendering"""
        job = self.crontab.new('addition1')
        job.minute.during(0, 3)
        job.hour.during(21, 23).every(1)
        job.dom.every(1)

        self.assertEqual(job.render(), '0,1,2,3 21,22,23 * * * addition1')

    def test_02_addition(self):
        """New Job Rendering"""
        job = self.crontab.new(command='addition2')

        job.minute.during(4, 9)
        job.hour.during(2, 10).every(2)
        job.dom.every(10)

        self.assertNotEqual(job.render(), '4-9 2-10/2 */3 * * addition2')
        self.assertEqual(job.render(), '4,5,6,7,8,9 2,4,6,8,10 1,11,21,31 * * addition2')

    def test_03_specials(self):
        """Ignore Special Symbols"""
        tab = crontab.CronTab(tabfile=os.path.join(TEST_DIR, 'data', 'specials.tab'))
        self.assertEqual(tab.render(), """0 * * * * hourly
0 0 * * * daily
0 0 * * 0 weekly
""")

    def test_04_comments(self):
        """Comments should be on their own lines"""
        self.assertEqual(self.crontab[0].comment, 'First Comment')
        self.assertEqual(self.crontab.render(), INITAL_TAB)
        job = self.crontab.new('command', comment="Test comment")
        self.assertEqual(job.render(), "# Test comment\n* * * * * command")

    def test_05_ansible(self):
        """Crontab shouldn't break ansible cronjobs"""
        cron = crontab.CronTab(tab="""
#Ansible: {job_name}
* * * * * {command}
""")
        self.assertEqual(cron[0].comment, '{job_name}')
        self.assertEqual(cron[0].command, '{command}')
        self.assertEqual(str(cron[0]), '#Ansible: {job_name}\n* * * * * {command}')

    def test_06_escaped_chars(self):
        """Do escaped chars parse correctly when read in"""
        cron = crontab.CronTab(tab="""
* * * * * cmd arg_with_\#_character # comment
""")
        self.assertEqual(cron[0].command, 'cmd arg_with_\\#_character')
        self.assertEqual(cron[0].comment, 'comment')

    def test_07_non_posix_shell(self):
        """Shell in windows environments is split correctly"""
        from crontab import open_pipe
        winfile = os.path.join(TEST_DIR, 'data', "bash\\win.exe")
        pipe = open_pipe("{sys.executable} {winfile}".format(winfile=winfile, sys=sys), 'SLASHED', posix=False)
        self.assertEqual(pipe.wait(), 0, 'Windows shell command not found!')
        (out, err) = pipe.communicate()
        self.assertEqual(out, b'Double Glazing Installed:SLASHED\n')
        self.assertEqual(err, b'')


if __name__ == '__main__':
    test_support.run_unittest(
       CompatTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2013 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab ranges.
"""
import unittest
from crontab import CronTab, CronSlice, PY3
try:
    from test import test_support
except ImportError:
    from test import support as test_support

from .utils import LoggingMixin

if PY3:
    unicode = str #pylint: disable=redefined-builtin

class RangeTestCase(LoggingMixin, unittest.TestCase):
    """Test basic functionality of crontab."""
    log_name = 'crontab'

    def setUp(self):
        super(RangeTestCase, self).setUp()
        self.crontab = CronTab(tab="")

    def test_01_atevery(self):
        """At Every"""
        tab = CronTab(tab="""
*  *  *  *  * command
61 *  *  *  * command
*  25 *  *  * command
*  *  32 *  * command
*  *  *  13 * command
*  *  *  *  8 command
        """)
        self.assertEqual(len(tab), 1)
        self.assertLog("error", "'61', not in 0-59 for Minutes")
        self.assertLog("error", "'25', not in 0-23 for Hours")
        self.assertLog("error", "'32', not in 1-31 for Day of Month")
        self.assertLog("error", "'13', not in 1-12 for Month")
        self.assertLog("error", "'8', not in 0-6 for Day of Week")

    def test_02_withinevery(self):
        """Within Every"""
        tab = CronTab(tab="""
*    *    *    *    * command
1-61 *    *    *    * command
*    1-25 *    *    * command
*    *    1-32 *    * command
*    *    *    1-13 * command
*    *    *    *    1-8 command
        """)
        self.assertEqual(len(tab), 1)
        self.assertLog("error", "'61', not in 0-59 for Minutes")
        self.assertLog("error", "'25', not in 0-23 for Hours")
        self.assertLog("error", "'32', not in 1-31 for Day of Month")
        self.assertLog("error", "'13', not in 1-12 for Month")
        self.assertLog("error", "'8', not in 0-6 for Day of Week")

    def test_03_outevery(self):
        """Out of Every"""
        tab = CronTab(tab="""
*    *    *    *    *   command
*/61 *    *    *    *   command
*    */25 *    *    *   command
*    *    */32 *    *   command
*    *    *    */13 *   command
*    *    *    *    */8 command
        """)
        self.assertEqual(len(tab), 1)
        self.assertLog("error", "'61', not in 0-59 for Minutes")
        self.assertLog("error", "'25', not in 0-23 for Hours")
        self.assertLog("error", "'32', not in 1-31 for Day of Month")
        self.assertLog("error", "'13', not in 1-12 for Month")
        self.assertLog("error", "'8', not in 0-6 for Day of Week")

    def test_03_inevery(self):
        """Inside of Every"""
        tab = CronTab(tab="""
*    *    *    *    *   command
*/59 *    *    *    *   command
*    */23 *    *    *   command
*    *    */30 *    *   command
*    *    *    */11 *   command
*    *    *    *    */7 command
        """)
        self.assertEqual(len(tab), 6, str(tab))

    def test_04_zero_seq(self):
        """Zero divisor in range"""
        tab = CronTab(tab="""
*/0 * * * * command
        """)
        self.assertEqual(len(tab), 0)
        self.assertLog("error", "Sequence can not be divided by zero or max")

    def test_14_invalid_range(self):
        """No numerator in range"""
        tab = CronTab(tab="/10 * * * * command")
        self.assertEqual(len(tab), 0)
        with self.assertRaises(ValueError):
            tab.render(errors=True)
        self.assertEqual(unicode(tab), "# DISABLED LINE\n# /10 * * * * command\n")
        self.assertLog('error', u"No enumeration for Minutes: '/10'")

    def test_05_sunday(self):
        """Test all possible day of week combinations"""
        for (a, b) in (\
              ("7", "0"), ("5-7", "0,5-6"), ("1-7", "*"), ("*/7", "0"),\
              ("0-6", "*"), ("2-7", "0,2-6"), ("1-5", "1-5"), ("0-5", "0-5")):
            v = str(CronSlice(4, a))
            self.assertEqual(v, b, "%s != %s, from %s" % (v, b, a))

    def test_06_backwards(self):
        """Test backwards ranges for error"""
        tab = CronTab(tab="* * * * 3-1 command")
        self.assertEqual(str(tab), "# DISABLED LINE\n# * * * * 3-1 command\n")
        self.assertLog("error", "Bad range '3-1'")

if __name__ == '__main__':
    test_support.run_unittest(RangeTestCase)
